package com.cesgroup.kingkong.service.impl;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.ParserConfig;
import com.cesgroup.common.utils.PatentTypeEnum;
import com.cesgroup.common.utils.TypeEnum;
import com.cesgroup.kingkong.mapper.EsMapper;

@Service
public class QueryService {

	@Autowired
	private EsMapper esMapper;


	/**根据id查询专利
	 * @param id
	 * @return
	 * Map<String,Object>
	 */
	public Map<String, Object> getById(String id) {
		Map<String, Object> result = new HashMap<String, Object>();
		// 首先id满足条件，通过Id去查
        // 匹配标准ID
		String VALID_NAME = "^[A-Za-z][A-Za-z].*[.][0-9]$";
		// 前缀正则：匹配前两位为英文字符
		String PRE_VALID_NAME = "^[A-Za-z][A-Za-z].*$";
		long ti = System.currentTimeMillis();
		System.out.println("开始查询：" + ti);
		//复合zl.23232.3这种标准格式的
		if (isValidId(id, VALID_NAME)) {
			//如果id是以zl开头的，把zl替换成CN
			if ("zl".equalsIgnoreCase(id.substring(0, 2))) {
				id = "CN" + id.substring(2, id.length());
			}
			//不是标准形式，但id前缀是以字母开头的
		} else if (isValidId(id, PRE_VALID_NAME)) {
			if ("zl".equalsIgnoreCase(id.substring(0, 2))) {
				id = "CN" + id.substring(2, id.length());
			}
			if (id.lastIndexOf(".") != -1) {
				id = id.substring(0, id.lastIndexOf("."));
			}
			//以上两种情况都不满足，没有前缀的情况（纯数字）
		} else {
			// 如果id中有点，先切割点之前的数字，然后拼接成CN+数字方式
			if (id.lastIndexOf(".") != -1) {
				id = id.substring(0, id.lastIndexOf("."));
			}
			//默认，直接将id字符串拼接成CN
			id = "CN".concat(id);
		}
		result = queryById(id);
		if (result.size() == 0 && id.lastIndexOf(".") != -1) {
			id = id.substring(0, id.lastIndexOf("."));
			result = queryById(id);
		}
		return result;
	}

	/**
	 * @Description: 查询论文信息
	 * @Param: [params]
	 * @return: java.util.Map<java.lang.String,java.lang.Object>
	 * @Author: Wanjianbin
	 * @Date: 2019/9/25
	 */
	public Map<String, Object> queryArticleInfo(Map<String, Object> params) {
		int start = 0;
		int count = 10;
		StringBuilder queryStr = new StringBuilder();

		// 分页参数
		Integer current = MapUtils.getInteger(params, "current");
		if (current != null) {
			start = (current - 1) * 10;
		}

		// 组合查询条件字符串
		String authorName = MapUtils.getString(params, "authorName");
		if (StringUtils.isNotEmpty(authorName)) {
			queryStr.append("AUTHOR-NAME(");
			queryStr.append(authorName);
			queryStr.append(")");
		}

		String affilorg = MapUtils.getString(params, "affilorg");
		if (StringUtils.isNotEmpty(affilorg)) {
			if (StringUtils.isNotEmpty(authorName)) {
				queryStr.append(" AND ");
			}
			queryStr.append("AFFILORG(");
			queryStr.append(affilorg);
			queryStr.append(")");
		}

		HashMap<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("start", start);
		paramMap.put("count", count);
		paramMap.put("query", queryStr.toString());

		String resultJsonString = esMapper.getArticleInfo(paramMap);
		Map<String, Object> result = parseArticleInfo(resultJsonString);
		if (result != null) {
			List<Map> list = (List<Map>) result.get("entry");
			if (list.get(0).get("error") != null) {
				return null;
			}
			return result;
		}
		return null;
	}

	/**
	 * 解析论文信息
	 * 
	 * @param articleInfo
	 * @return Map<String,Object>
	 */
	private Map<String, Object> parseArticleInfo(String articleInfo) {
		if (StringUtils.isNotEmpty(articleInfo)) {
		    // JSON解析配置
			ParserConfig.getGlobalInstance().setAutoTypeSupport(true);
			Map<String, Object> result = (Map<String, Object>) JSONObject.parse(articleInfo);
			// 抽出search-results信息
			return (Map<String, Object>) result.get("search-results");
		}
		return null;
	}

	/**
	 * id格式检验
	 * 
	 * @param name
	 * @param regx
	 * @return boolean
	 */
	private boolean isValidId(String name, String regx) {
		Pattern pattern = Pattern.compile(regx);
		return (name != null) && pattern.matcher(name.replaceAll(" ", "")).matches();
	}

	/**
	 * 根据id查询到所有结果
	 * 
	 * @param id
	 * @return Map<String,Object>
	 */
	public Map<String, Object> queryById(String id) {
		// 需要返回的字段
		String fields = "[\"pubkind\",\"title\",\"legal_status_result\",\"appno\",\"appdate\",\"pubno\",\"pubdate\",\"legal_status\",\"granted_status\",\"patent_type\",\"ipcs\",\"agents\",\"inventors\",\"applicants\",\"owners\",\"abstract\"]";
		Map<String, Object> resultMap = new HashMap<String, Object>();
		Map<String, Object> result = new HashMap<String, Object>();
		JSONObject sourceJsonObject = null;
		String query = new String("{\"_source\":" + fields + ",\"query\":{\"bool\":{\"must\":[{\"prefix\":{\"appno\":"
				+ "\"" + id + "\"" + "}}]}}}");
		String resultString = esMapper.postQuery(query);
		if (StringUtils.isNotBlank(resultString)) {
			JSONObject hitJsonObject = JSONObject.parseObject(resultString).getJSONObject("hits");
			JSONArray hitsArray = (JSONArray) hitJsonObject.get("hits");
			if (hitsArray.size() > 0) {
				sourceJsonObject = hitsArray.getJSONObject(0).getJSONObject("_source");
				result = parseResult(sourceJsonObject);
				if (null != result) {
					// 查询成功，有结果返回
					resultMap.put("resonCode", "200");
					resultMap.put("result", result);
				} else {
					// 查询成功，无结果返回，数据模型解析错误
					resultMap.put("resonCode", "2012");
					resultMap.put("result", "");
				}
			} else {
				// 查询成功，没有结果返回
				resultMap.put("resonCode", "2011");
				resultMap.put("result", "");
			}
		} else {
			// 1011 查询超时
			resultMap.put("resonCode", "1011");
			resultMap.put("result", "");
		}
		return resultMap;
	}

	private Map<String, Object> parseResult(JSONObject sourceJsonObject) {
		Map<String, Object> resultMap = new HashMap<String, Object>();
		String cnTitle = sourceJsonObject.getJSONObject("title").getString("text_en");
		String enTitle = sourceJsonObject.getJSONObject("title").getString("text_zh");
		resultMap.put("title", StringUtils.isNotBlank(cnTitle) ? cnTitle : enTitle);
		String application_number = sourceJsonObject.getString("appno");
		resultMap.put("application_number", application_number);
		String application_date1 = sourceJsonObject.getString("appdate");
		String application_date = "";
		if (StringUtils.isNotBlank(application_date1)) {
			application_date = application_date1.substring(0, 4) + "-" + application_date1.substring(4, 6) + "-"
					+ application_date1.substring(6, 8);
		}
		resultMap.put("application_date", application_date);
		String pubno = sourceJsonObject.getString("pubno");
		String pubkind = sourceJsonObject.getString("pubkind");
		JSONObject pmJsonObject = new JSONObject();
		pmJsonObject.put("pubno", StringUtils.isNotBlank(pubno) ? pubno : "");
		pmJsonObject.put("pubkind", StringUtils.isNotBlank(pubkind) ? pubkind : "");
		resultMap.put("publication_number", pmJsonObject);
		String publication_date1 = sourceJsonObject.getString("pubdate");
		String publication_date = "";
		if (StringUtils.isNotBlank(publication_date1)) {
			publication_date = publication_date1.substring(0, 4) + "-" + publication_date1.substring(4, 6) + "-"
					+ publication_date1.substring(6, 8);
		}
		resultMap.put("publication_date", publication_date);
		JSONObject legalStatusJsonObject = sourceJsonObject.getJSONObject("legal_status_result");
		resultMap.put("legal_status",
				null != legalStatusJsonObject ? legalStatusJsonObject.getString("legalstatus") : "");
		resultMap.put("granted_status",
				null != legalStatusJsonObject ? legalStatusJsonObject.getString("granted_status") : "");
		String kind_code_desc = sourceJsonObject.getString("patent_type");
		resultMap.put("kind_code_desc", kind_code_desc);
		JSONObject ipcJsonObject = sourceJsonObject.getJSONObject("ipcs");
		if (null != ipcJsonObject) {
			JSONArray IPC_list = ipcJsonObject.getJSONArray("mainclass");
			List<String> ipcNames = new ArrayList<String>();
			if (null != IPC_list) {
				for (int j = 0; j < IPC_list.size(); j++) {
					String nameString = null != IPC_list.getString(j)
							? null != PatentTypeEnum.parse(IPC_list.getString(j))
									? PatentTypeEnum.parse(IPC_list.getString(j)).getMessage().replaceAll(" ", "")
									: ""
							: "";
					ipcNames.add(nameString);
				}
				resultMap.put("IPC_list", IPC_list);
				resultMap.put("IPC_desc", ipcNames);
			} else {
				resultMap.put("IPC_list", "");
				resultMap.put("IPC_desc", "");
			}
		} else {
			resultMap.put("IPC_list", "");
			resultMap.put("IPC_desc", "");
		}
		String appcountry = StringUtils.isNoneBlank(sourceJsonObject.getString("appno"))
				? sourceJsonObject.getString("appno").substring(0, 2)
				: "";
		resultMap.put("appcountry",
				null != TypeEnum.parse(appcountry) ? TypeEnum.parse(appcountry).getMessage().replaceAll(" ", "") : "");

		JSONArray agengtObjectArray = sourceJsonObject.getJSONArray("agents");
		List<String> newAgengts = new ArrayList<String>();
		for (int i = 0; i < agengtObjectArray.size(); i++) {
			JSONObject inventorsObject = agengtObjectArray.getJSONObject(i);
			String agency = inventorsObject.getString("agency");
			newAgengts.add(agency);
		}
		resultMap.put("agency", newAgengts);
		JSONArray inventorObjectArray = sourceJsonObject.getJSONArray("inventors");
		List<String> newInventors = new ArrayList<String>();
		for (int i = 0; i < inventorObjectArray.size(); i++) {
			JSONObject inventorsObject = inventorObjectArray.getJSONObject(i);
			String nameString = null != inventorsObject ? StringUtils.isNotBlank(inventorsObject.getString("name_zh"))
					? inventorsObject.getString("name_zh")
					: inventorsObject.getString("name_en") : "";
			newInventors.add(nameString);
		}
		resultMap.put("inventors", newInventors);
		JSONArray applicantsArray = sourceJsonObject.getJSONArray("applicants");
		List<Map<String, Object>> newApplicants = new ArrayList<Map<String, Object>>();
		for (int i = 0; i < applicantsArray.size(); i++) {
			Map<String, Object> assigneesMap = new HashMap<String, Object>();
			JSONObject assigneeObject = applicantsArray.getJSONObject(i);
			assigneesMap.put("names",
					null != assigneeObject ? StringUtils.isNoneBlank(assigneeObject.getString("name_zh"))
							? assigneeObject.getString("name_zh")
							: assigneeObject.getString("name_en") : "");
			if (null != assigneeObject) {
				JSONObject zhJsonObject = assigneeObject.getJSONObject("address_zh");
				JSONObject enJsonObject = assigneeObject.getJSONObject("address_en");
				if (null != zhJsonObject) {
					assigneesMap.put("province",
							StringUtils.isNotBlank(zhJsonObject.getString("province"))
									? zhJsonObject.getString("province")
									: "");
					assigneesMap.put("city",
							StringUtils.isNotBlank(zhJsonObject.getString("city")) ? zhJsonObject.getString("city")
									: "");
				} else if (null != enJsonObject) {
					assigneesMap.put("province",
							StringUtils.isNotBlank(enJsonObject.getString("province"))
									? enJsonObject.getString("province")
									: "");
					assigneesMap.put("city",
							StringUtils.isNotBlank(enJsonObject.getString("city")) ? enJsonObject.getString("city")
									: "");
				} else {
					assigneesMap.put("province", "");
					assigneesMap.put("city", "");
				}
			} else {
				assigneesMap.put("province", "");
				assigneesMap.put("city", "");
			}
			newApplicants.add(assigneesMap);
		}

		resultMap.put("assignees", newApplicants);

		JSONArray ownersObjectArray = sourceJsonObject.getJSONArray("owners");
		List<Map<String, Object>> newOwners = new ArrayList<Map<String, Object>>();
		for (int i = 0; i < ownersObjectArray.size(); i++) {
			JSONObject ownersObject = ownersObjectArray.getJSONObject(i);
			Map<String, Object> ownerMap = new HashMap<String, Object>();
			ownerMap.put("names",
					null != ownersObject ? StringUtils.isNotBlank(ownersObject.getString("name_zh"))
							? ownersObject.getString("name_zh")
							: ownersObject.getString("name_en") : "");
			if (null != ownersObject) {
				JSONObject ownerZhJsonObject = ownersObject.getJSONObject("address_zh");
				JSONObject ownerEnJsonObject = ownersObject.getJSONObject("address_en");
				if (null != ownerZhJsonObject) {
					ownerMap.put("province",
							StringUtils.isNoneBlank(ownerZhJsonObject.getString("province"))
									? ownerZhJsonObject.getString("province")
									: "");
					ownerMap.put("city",
							StringUtils.isNoneBlank(ownerZhJsonObject.getString("city"))
									? ownerZhJsonObject.getString("city")
									: "");
				} else if (null != ownerEnJsonObject) {
					ownerMap.put("province",
							StringUtils.isNoneBlank(ownerEnJsonObject.getString("province"))
									? ownerEnJsonObject.getString("province")
									: "");
					ownerMap.put("city",
							StringUtils.isNoneBlank(ownerEnJsonObject.getString("city"))
									? ownerEnJsonObject.getString("city")
									: "");
				} else {
					ownerMap.put("province", "");
					ownerMap.put("city", "");
				}
			} else {
				ownerMap.put("province", "");
				ownerMap.put("city", "");
			}
			newOwners.add(ownerMap);
		}
		if (newOwners.size() > 0) {
			resultMap.put("owners", newOwners);
		} else {
			try {
				ByteArrayOutputStream bo = new ByteArrayOutputStream();
				ObjectOutputStream oo = new ObjectOutputStream(bo);
				oo.writeObject(newApplicants);
				// 从流里读出来
				ByteArrayInputStream bi = new ByteArrayInputStream(bo.toByteArray());
				ObjectInputStream oi = new ObjectInputStream(bi);
				Object aObject = oi.readObject();
				resultMap.put("owners", aObject);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		JSONObject abstrJsonObject = sourceJsonObject.getJSONObject("abstract");
		String abstract1 = "";
		if (null != abstrJsonObject) {
			abstract1 = StringUtils.isNotBlank(abstrJsonObject.getString("text_zh"))
					? abstrJsonObject.getString("text_zh")
					: abstrJsonObject.getString("text_en");
		}
		resultMap.put("abstract", abstract1);
		return resultMap;
	}

}
